var gulp = require('gulp');
var concat = require('gulp-concat');
var less = require ('gulp-less');
var uglyfly = require('gulp-uglyfly');
var browserSync = require ('browser-sync').create();
var pug	= require('gulp-pug');
var rename = require('gulp-rename');
var minifyCss = require('gulp-minify-css');
var plumber = require('gulp-plumber');


// Origin
var jsFiles = ['src/js/*.js'];
var lessFiles = ['src/less/*.less'] ;
var mainLessFile = 'src/less/main.less';
var fontAwesomeFIles = ['src/font-awesome/fonts/*'];
var testFiles = 'src/test/pug/*.pug';
var testAppFiles = 'src/test/pug/*.js';

// Destination
var distFolder = 'dist/';
var htmlTestDestination = 'src/test/html/';
var fontAwesomeDestination = 'plugins/font-awesome/'

// Destination file names
var jsDistFullName = 'masterrow.js';
var jsDistMinName = 'masterrow.min.js';
var cssDistFullName = 'masterrow.css';
var cssDistMinName = 'masterrow.min.css';


// Tasks
// Move the font-awesome folder to the plugin folder. Just need to work once.
gulp.task('fontAwesomeProcess', function (){
	return gulp.src(fontAwesomeFIles)
		.pipe(gulp.dest(distFolder + fontAwesomeDestination))
	    .pipe(browserSync.stream())
});

// Read the JS files and concat to the destination
gulp.task('jsProcess', function (){

	return gulp.src(jsFiles)
		.pipe(plumber())
		.pipe(concat(jsDistFullName))
		.pipe(gulp.dest(distFolder))
	    .pipe(browserSync.stream())
});

// Same process above, but minify
gulp.task('jsMinifyProcess', function (){

	return gulp.src(jsFiles)
		.pipe(plumber())
		.pipe(concat(jsDistMinName))
		.pipe(uglyfly())
		.pipe(gulp.dest(distFolder))
	    .pipe(browserSync.stream())
});


// Read the LESS files, process to the CSS and concat to the destination
gulp.task('lessProcess', function (){
	console.log('Less Process');
	return gulp.src(mainLessFile)
		.pipe(less())
		.pipe(rename(cssDistFullName))
		.pipe(gulp.dest(distFolder))
	    .pipe(browserSync.stream())
});


// Same above, but minifying
gulp.task('lessMinifyProcess', function (){
	return gulp.src(mainLessFile)
		.pipe(less())
		.pipe(minifyCss())
		.pipe(rename(cssDistMinName))
		.pipe(gulp.dest(distFolder))
	    .pipe(browserSync.stream())
});


// Process the pug files into Html 
gulp.task('pugProcess', function (){
	
	return gulp.src(testFiles)
		.pipe(plumber())
		.pipe(pug({
			pretty: true
		}))
		.pipe( gulp.dest(htmlTestDestination))
		.pipe(browserSync.stream());
});

// Copy the JS file of test to the HTMl folder
gulp.task('jsTestProcess', function (){
	
	return gulp.src(testAppFiles)
		.pipe(plumber())
		.pipe( gulp.dest(htmlTestDestination))
		.pipe(browserSync.stream());
});


// Watchers
gulp.task ('watch', function(){
	
	gulp.watch(jsFiles, ['jsProcess', 'jsMinifyProcess']);
	gulp.watch(lessFiles, ['lessProcess', 'lessMinifyProcess']);	
	gulp.watch(testFiles, ['pugProcess']);
	gulp.watch(testAppFiles, ['jsTestProcess']);
});


// The browser refresh automatization
gulp.task('browser-sync', function(){

	browserSync.init({
		proxy: "localhost/MasterRow/src/test/html/test.html",
		notify: false
	});

});


// The task groups
gulp.task( 'default', ['jsProcess', 'jsMinifyProcess', 'fontAwesomeProcess', 'lessProcess', 'jsTestProcess', 'lessMinifyProcess', 'pugProcess', 'watch', 'browser-sync']);